
# Copyright (C)  2004 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
# Silke Reimer <silke@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.




import sys
import edbsclasses
import logging
log = logging.getLogger('')

# begin of the preparsing
def first_parsing(file):
    """Sucht nach zusammengehrigen EDBS-Zeilen; sucht die EDBS-Datenstze
    durch und bearbeitet sie je nach Zugehrigkeitsbyte:
    ' ': ist, wird ein Eintrag edbs_satz in plist angehngt
    'A' ist, wird eine edbs_satz Objekt mit edbs_satz dieses Satzes und der
        Folgestze gefllt und an plist angehngt.
    Folgestze sind EDBS-Stze mit Zugehrigkeitsbyte 'F' oder am Ende 'E'.
    """

    plist = []
    line = file.readline()

    if line !="":
        entry_tmp = edbsclasses.edbs_satz(line)

    while line != "":
        if entry_tmp.relation() == " ":
            plist += [entry_tmp]
        elif entry_tmp.relation() == "A":
            list_tmp =edbsclasses.edbs_satz_liste(entry_tmp)
            line = file.readline()
            if line =="":
                raise "error: Folge- oder End-EDBS-Satz erwartet" 
            entry_tmp = edbsclasses.edbs_satz(line)      
            while entry_tmp.relation() == "F":
                list_tmp.additionalentry(entry_tmp)
                line =file.readline()
                entry_tmp = edbsclasses.edbs_satz(line)
            if entry_tmp.relation() == "E":
                list_tmp.additionalentry(entry_tmp)
            else:
                 raise "error: Andere Zugehoerigkeit erwartet"  
            plist += [ list_tmp]
        else :
            raise "error: Falsche Zugehoerigkeit"

        line =file.readline()
        entry_tmp = edbsclasses.edbs_satz(line)

    return plist

def second_parsing(ONS, eliste):     
    """Geht die edbs_satz/edbs_satzliste Liste elist durch und checkt get_entryart
    und verarbeitet den Eintrag der Art entsprechend:
        Art:                  | Was gemacht wir:
        OBJEKTDATEN             Eintrag in den Objektdaten_sammler
        AUFTRAGSKENNDATEN       Eintrag ins Logbuch
        AUFTRAGSTEXTDATEN       Eintrag ins Logbuch
        AENDERUNSDATUM          Eintrag ins Logbuch (nur BZSN)
        OBJLOESCHEN             Eintrag in den Objektdaten_sammler (fr BZSN)
        GEBIETSKENNZEICHNUNG    Pkte werden umgewandelt und ins Logbuch abgelegt
        AUFTRAGSKENNSATZ        Eintrag ins Logbuch
        AUFTRAGSENDE            Eintrag ins Logbuch als "Ende"
        UNDEFINIERT             Eintrag ins Logbuch als "Undefiniert"

    Die Routine gibt den Objektsammelspeicher (ONS) zurck.         
    """

    for e in eliste:
        art = e.get_entryart()

        if art == "OBJEKTDATEN":
            II_daten = edbsclasses.II_data(e.get_II_data())
            II_typ = II_daten.get_OIoderL()

            #Objekt mit gamma-Teil
            if II_typ == "O": 
                #geht alle gamma-Teile durch
                for  gamma in range(II_daten.get_gammalen()): 
                    Objnr = II_daten.get_FktdesObj(gamma)[10:17]
                    if not ONS.has_key(Objnr):
                        ONS[Objnr]= edbsclasses.objdaten_sammler(Objnr)
                    ONS[Objnr].setObjdaten(e.get_II_data()[2][gamma],e.get_II_data()[0])

            elif II_typ == "L":
                for alpha in range(II_daten.get_alphalen()):
                    objKoor = II_daten.get_Objektkoor()
                    endPkt = II_daten.get_EndpktderLinie(alpha)
                    art = II_daten.get_ArtderLiniengeo(alpha)
                    lageparam = II_daten.get_Lageparaliste(alpha)
                    linienFkt = II_daten.get_FktderLinieliste(alpha)

                    for theta in linienFkt:

                        #holte die linke und die rechte Objnr aus II_daten
                        ObjnrL = theta[7:14] 
                        ObjnrR = theta[14:21]

                        # Trage Lageparameter in ONS zur Objnr ein, wenn
                        # notwendig lege ein neues Objekt im ONS an.
                        if ObjnrL != '       ': 
                            tmp_line = edbsclasses.edbs_line(
                                [objKoor, endPkt, art, lageparam,
                                theta[0:3], theta[3:7], theta[21]],'N')
                            if not ONS.has_key(ObjnrL):
                                ONS[ObjnrL] = edbsclasses.objdaten_sammler(ObjnrL)
                            ONS[ObjnrL].appendline(tmp_line)
                        if ObjnrR != '       ': 
                            tmp_line = edbsclasses.edbs_line(
                                [objKoor, endPkt, art, lageparam,
                                theta[0:3], theta[3:7], theta[22]],'R')
                            if not ONS.has_key(ObjnrR):
                                ONS[ObjnrR] = edbsclasses.objdaten_sammler(ObjnrR)
                            ONS[ObjnrR].appendline(tmp_line)

            else:
                raise "error : seltsamer Mischtyp von Daten"

        elif art =="AUFTRAGSKENNDATEN":
            II_text = e.get_II()
            log.info("Auftragskenndaten:\n")
            kenndaten = II_text[4:].split(' - ')
            for k in kenndaten:
                log.info("    %s\n" % k.strip())
        elif art == "AUFTRAGSTEXTDATEN":
            II_text = e.get_II()
            log.info("Auftragstextdaten:\n")
            log.info(II_text[4:]) 
        elif art == "AENDERUNSDATUM":
            II_text = e.get_II()
            log.info("nderung am: %s.%s.%s " % (II_text[4:6], 
                                                     II_text[2:4],
                                                     II_text[0:2]))
            log.info("um %s:%s:%s\n" % (II_text[7,9], 
                                             II_text[9,11], 
                                             II_text[11,13]))
        elif art== "OBJDATENLOESCHEN":
            (Objnr, AK) = e.get_SK_Objnr()
            if not ONS.has_key(Objnr):
                ONS[Objnr] = edbsclasses.objdaten_sammler(Objnr)
            ONS[Objnr].loescheObj(AK)

        elif art == "GEBIETSKENNZEICHNUNG":
            II_text = e.get_II()
            log.info("Gebietskennzeichen: \n")
            log.info("    linksunten: (%f, %f )\n" 
                        % edbsclasses.punkttrans(II_text[ 0:20]))
            log.info("    rechtsoben: (%f, %f )\n" 
                        % edbsclasses.punkttrans(II_text[20:40]))

        elif art == "AUFTRAGSKENNSATZ":
            II_text = e.get_II()
            log.info("Auftragskennsatz: \n")
            log.info("Dienststelle: \n")
            log.info("    %s\n"% II_text[4:18] ) # Holt die Dienststelle raus
            log.info("    %s\n" %II_text[18: 51])
            log.info("Ausgabeinfo: \n" )
            log.info("    %s\n" %II_text[51:83].strip())
            log.info("Rest des Eintrags: ")
            log.info("    %s\n" %II_text[83:])

        elif art == "AUFTRAGSENDE":
            log.info("Ende\n")

        elif art == "UNDEFINIERT":
            log.info("Undefiniert:\n")
            log.info(str(e))

        else:
            raise "error: Unerwarteter edbs_satz -Typ"
    return  ONS

def parsen(ONS, Quelle):    
    """Auslesen eine EDBS-Datei (Quelle).
    Im ersten Schritt werden zusammengehrige EDBS-Satze zusammengefhrt.
    Der zweite Schritt liest die Objekte aus einem EDBS-Auftragssatz und
    legt die Daten in einem Objektdaten-Sammler ab.
    """

    # ffnen der Quellendatei
    quell = open(Quelle ,"r") 
    # Logbucheintrag schreiben: der Header
    log.info("================================================\n")
    log.info("   %s \n" % Quelle)
    log.info("================================================\n")
    # parsen der Daten     
    liste = first_parsing(quell)
    return second_parsing(ONS, liste)

